package com.pj.face.handler.impl;

import cn.hutool.log.StaticLog;
import com.pj.face.brand.FaceBrand;
import com.pj.face.handler.IFaceHandler;
import com.pj.face.handler.VerifyVO;
import com.pj.face.properties.TencenProperties;
import com.pj.utils.sg.AjaxError;
import com.tencentcloudapi.common.Credential;
import com.tencentcloudapi.common.exception.TencentCloudSDKException;
import com.tencentcloudapi.common.profile.ClientProfile;
import com.tencentcloudapi.common.profile.HttpProfile;
import com.tencentcloudapi.iai.v20200303.IaiClient;
import com.tencentcloudapi.iai.v20200303.models.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.atomic.AtomicReference;

/**
 * 人脸库管理地址
 * https://console.cloud.tencent.com/aiface/face-lib
 */
@Service
@Slf4j
public class TencenFaceHandlerImpl implements IFaceHandler {
    @Resource
    private TencenProperties tencenProperties;

    @Override
    public VerifyVO verify(String personId, String baseImage) {
        VerifyVO vo = new VerifyVO();
        vo.setSuccess(false);
        IaiClient client = this.createClient();
        VerifyPersonRequest req = new VerifyPersonRequest();
        req.setImage(baseImage);
        req.setPersonId(personId);
        // 返回的resp是一个VerifyPersonResponse的实例，与请求对象对应
        VerifyPersonResponse resp;
        try {
            resp = client.VerifyPerson(req);
        } catch (TencentCloudSDKException e) {
            log.error("verify person : {} fail,{}", personId, e.getMessage());
            StaticLog.info("verify person : {} fail,{}", personId, e.getMessage());
            vo.setMsg(e.getMessage());
            return vo;
        }
        boolean result=resp != null && resp.getIsMatch();
        vo.setMsg(result?"成功":"不匹配");
        vo.setSuccess(result);
        return vo;
    }

    /**
     * 查找人脸库
     *
     * @param baseImg
     * @return
     */
    public List<String> faceSearch(String baseImg) {
        StaticLog.info("face:{}", baseImg);
        IaiClient client = this.createClient();
        SearchFacesRequest request = new SearchFacesRequest();
        request.setImage(baseImg);
        request.setMaxFaceNum(1L);
        request.setMinFaceSize(80L);
        request.setMaxPersonNum(2L);
        request.setGroupIds(new String[]{tencenProperties.getGroupId()});
        SearchFacesResponse response;
        List<String> faceId = new ArrayList<>();
        try {
            response = client.SearchFaces(request);
        } catch (TencentCloudSDKException e) {
            e.printStackTrace();
            StaticLog.error("face error: " + e.getMessage());
            throw new AjaxError("无人脸信息");
        }
        Arrays.stream(response.getResults())
                .forEach(result -> {
                    Arrays.stream(result.getCandidates())
                            .filter(candidate -> candidate.getScore() >= 80)
                            .forEach(candidate -> faceId.add(candidate.getPersonId()));
                });
        return faceId;
    }

    public void createPersonByBase(String personId, String name, String imageBase) {
        IaiClient client = this.createClient();
        // 实例化一个请求对象,每个接口都会对应一个request对象
        CreatePersonRequest req = new CreatePersonRequest();
        req.setGroupId(tencenProperties.getGroupId());
        req.setPersonName(name);
        req.setPersonId(personId);
        req.setImage(imageBase);
        // 返回的resp是一个CreatePersonResponse的实例，与请求对象对应
        try {
            client.CreatePerson(req);
        } catch (TencentCloudSDKException e) {
            e.printStackTrace();
        }

    }

    public boolean checkIsLive(String baseImg) {
        IaiClient client = this.createClient();
        DetectLiveFaceRequest req = new DetectLiveFaceRequest();
        req.setImage(baseImg);
        DetectLiveFaceResponse resp;
        // 返回的resp是一个DetectLiveFaceResponse的实例，与请求对象对应
        try {
            resp = client.DetectLiveFace(req);
        } catch (TencentCloudSDKException e) {
            log.error("check live fail,{}", e.getMessage());
            throw new AjaxError(e.getMessage());
        }
        return resp != null && resp.getIsLiveness();
    }

    @Override
    public FaceBrand faceBrand() {
        return FaceBrand.TENCEN_SMS;
    }

    private IaiClient createClient() {
        // 实例化一个认证对象，入参需要传入腾讯云账户secretId，secretKey,此处还需注意密钥对的保密
        // 密钥可前往https://console.cloud.tencent.com/cam/capi网站进行获取
        Credential cred = new Credential(tencenProperties.getSecretId(), tencenProperties.getSecretKey());
        // 实例化一个http选项，可选的，没有特殊需求可以跳过
        HttpProfile httpProfile = new HttpProfile();
        httpProfile.setEndpoint("iai.tencentcloudapi.com");
        // 实例化一个client选项，可选的，没有特殊需求可以跳过
        ClientProfile clientProfile = new ClientProfile();
        clientProfile.setHttpProfile(httpProfile);
        // 实例化要请求产品的client对象,clientProfile是可选的
        return new IaiClient(cred, "ap-chongqing", clientProfile);
    }

}
